/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */
/*global $, window, location, CSInterface, SystemPath, themeManager*/

(function () {
    'use strict';

    var csInterface = new CSInterface();
    var currentSocket = null;
    var currectCompId = null;
    var marqeeText = '';
    
    function init() {
        document.getElementById('files').addEventListener('change', handleFileSelect, false);
        
        themeManager.init();
        
        var http = window.require('net');
        //Lets define a port we want to listen to
        const PORT=8883;

        //Create a server
        var server = http.createServer(function (socket) {
            console.log('Server started: Waiting for client connection ...');
            console.log('Client connected:port,address: %s, %j',(socket.remotePort,      socket.remoteAddress));
            currentSocket = socket;
            var addr = String(socket.remoteAddress).split(':').slice(-1)[0]
            console.log("Address: " + addr)
            if (addr !== '127.0.0.1'){
                var r = confirm("Accept incoming connection from: " + socket.remoteAddress);
                if (r == false) {
                    socket.end()
                    return;
                }
            }
            
            var fullData = '';
            socket.on('data', function (data) {
                $('.marquee').text("Receiving data...");
                data = String(data);
                fullData += data;
                var index = fullData.indexOf('>');
                while (index > 0) {
                    var msgLength = parseInt(fullData.substring(0, index));
                    if (fullData.length > index + msgLength){
                        
                        var msg = fullData.substring(index + 1, index + 1 + msgLength);
                        fullData = fullData.substring(index + 1 + msgLength, fullData.length);
                        
                        var jsData = JSON.parse(msg);
                        if (jsData.hasOwnProperty('handshake')){
                            csInterface.evalScript('getCurrentComposition(' + msg + ')', function (res) {
                                if (res.length==0) return;
                                socket.write(res.length + '>' + res);
                                var resData = JSON.parse(res);
                                currectCompId = resData.compId;
                                marqeeText = jsData.prefs.name + ' - ' + resData.compName;
                                $('.marquee').html(marqeeText);
                                $('.marquee').marquee({
                                    duration: 5000,
                                    startVisible: true,
                                    delayBeforeStart: 500,
                                    pauseOnHover: true
                                });
                                $('#status').css('color', '#ccc');
                                $('#statusCircleRed').css("display","none");
                                $('#statusCircleGreen').css("display","inline-block");
                            });
                            
                        } else {
                            
                            jsData.compId = currectCompId;
                            var interData = JSON.stringify(jsData);
                            csInterface.evalScript('initAE(' + interData + ')', function (res) {
                                if(res === 'requestHandshake'){
                                    if(currentSocket !== null ){
                                        var msg = 'Request handshake';
                                        currentSocket.write(msg.length + '>' + msg);
                                    }     
                                }
                            });
                            
                            $('.marquee').html(marqeeText);
                            $('.marquee').marquee({
                                duration: 5000,
                                startVisible: true,
                                delayBeforeStart: 500,
                                pauseOnHover: true
                            });
                            
                        }
                        index = -1;
                    }
                    var index = fullData.indexOf('>', index + 1);
                }
            });
            
            
            socket.on('end', function (data) {
                $('.marquee').text("Not connected");
                $('#status').css('color', '#777');
                $('#statusCircleRed').css("display","inline-block");
                $('#statusCircleGreen').css("display","none");
                console.log('Disconnected from server');
                currentSocket = null;
            });
        });

        //Lets start our server
        server.listen(PORT, function(){
            //Callback triggered when server is successfully listening. Hurray!
            console.log("Server listening on: http://localhost:%s", PORT);
            console.log("Server ip: %j", server.address() )
        });
        
        $('#importButton').click( function(){
            $('#files').trigger('click');         
        });
        
        $('#exportButton').click( function(){
            csInterface.evalScript('sendToMaya()', function (data) {  
                saveFile('#export_file',data);
            });      
        });
        
        
        $('#activeCompButton').click( function(){
            if(currentSocket !== null ){
                var msg = 'Request handshake';
                currentSocket.write(msg.length + '>' + msg);
            }      
        });
        
        
        $('#dropField').on(
            'dragover',
            function(e) {
                e.preventDefault();
                e.stopPropagation();
            }
        )
        $('#dropField').on(
            'dragenter',
            function(e) {
                e.preventDefault();
                e.stopPropagation();
            }
        )
                
        function saveFile(name,data) {
            var chooser = $(name);
            chooser.unbind('change');
            chooser.change(function(evt) {
                window.cep.fs.writeFile($(this).val(), data);
                $(this).val(null);
            });

            chooser.trigger('click'); 
          }
        
        function handleFileSelect(evt) {
            var f = evt.target.files[0];
            evt.target.value = null;
            if(typeof f==='undefined'){
                return;
            }
            var reader = new FileReader();
            reader.onload = (function(theFile) {
                return function(e) {
                    var data = e.target.result;
                    csInterface.evalScript('getCurrentComposition(' + data + ')', function (res) {  
                        if (res.length==0) return;
                        var resData = JSON.parse(res);
                        var jsData = JSON.parse(data);
                        jsData.compId = resData.compId;
                        var interData = JSON.stringify(jsData);
                        csInterface.evalScript('initAE(' + interData + ')', function (res) {});
                    });
                };
              })(f);
            if(f instanceof Blob){
                reader.readAsText(f);
            }

        }
        

        $('#dropField').on(
            'drop',
            function(e){
                if(e.originalEvent.dataTransfer){
                    csInterface.evalScript('sendToMaya()', function (data) {  
                        if(currentSocket === null ){
                            saveFile('#export_file',data);
                        }else{
                            currentSocket.write(data.length + '>' + data, null, function(){
                                $("#dropField").css('background-image', 'url(img/DragDrop.gif?' + new Date().getTime() + ')');
                            });
                        }
                    });
                    //$("#dropField").attr('background-image', 'url(../img/DragDrop.png)');
                    /*  //Handle Files
                    if(e.originalEvent.dataTransfer.files.length) {
                        e.preventDefault();
                        e.stopPropagation();
                        console.log(e.originalEvent.dataTransfer.files);
                    }  */
                    
                }
            }
        );
        
        
        
        $(document).ready(function(){
            $('img').tooltip({
     'delay': { show: 1000, hide: 600 }
});
	$("[rel='tooltip']").tooltip();
            
    })
        
        
    }
  
        
    init();

}());
    
