/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */
/*global $, Folder, app*/
#include "json2.js" // jshint ignore:line

function sendToMaya(){
    'use strict';
    
    var data = [];
    
    function getProperty(prop){
        p = {values: [], times: []};
        var minFrame = null;
        var maxFrame = null;
        for(var j = 0; j < prop.numKeys; ++j){
            var f = prop.keyTime(j+1);
            if(maxFrame === null || maxFrame < f)
                maxFrame = f;
            if(minFrame === null || minFrame > f)
                minFrame = f;
        }
        if(minFrame !== null && maxFrame !== null){
            for(var j = minFrame * myComp.frameRate; j < maxFrame * myComp.frameRate + 1; ++j){
                var timeInSeconds = j * 1.0 / myComp.frameRate;
                p.values.push(prop.valueAtTime(timeInSeconds, true));
                p.times.push(timeInSeconds);
            }
        }
        if(p.values.length == 0)
            p.values.push(prop.value)
        return p;
    }
    
    var myComp = app.project.activeItem; // our current selected & opened comp
    if(myComp && myComp.selectedLayers[0]){ // if a comp and at least one property is selected
        for(var i = 0 ; i < myComp.selectedLayers.length; i++){ // go trough each selected layers
            var currentLayer = myComp.selectedLayers[i];
            var obj = {};
            if(currentLayer instanceof CameraLayer){
                obj.type = 'camera';
                obj.zoom = getProperty(currentLayer.zoom);
                obj.compWidth = myComp.width;
            }else if(currentLayer instanceof LightLayer){
                obj.type = 'light';
                if(currentLayer.lightType == LightType.PARALLEL){
                    
                    obj.lightType = "directional";
                    
                }
                
                if(currentLayer.lightType == LightType.SPOT){
                    
                    obj.lightType = "spot";
                    
                    if(currentLayer.hasOwnProperty("coneAngle")){
                        obj.coneAngle = getProperty(currentLayer.coneAngle);
                    }
                }
                
                if(currentLayer.lightType == LightType.POINT){
                    obj.lightType = "point";
                }
                
                if(currentLayer.lightType == LightType.AMBIENT){
                    obj.lightType = "ambient";
                }
                
                if(currentLayer.hasOwnProperty("color")){
                    obj.color = getProperty(currentLayer.color);
                }
                
                if(currentLayer.hasOwnProperty("intensity")){
                    obj.intensity = getProperty(currentLayer.intensity);
                }
                
            }else if(currentLayer.nullLayer){
                obj.type = 'locator';
                if(currentLayer.hasOwnProperty("scale"))
                    obj.scale = getProperty(currentLayer.scale);
            }else if(currentLayer.hasOwnProperty("source") && 
                     currentLayer.source.hasOwnProperty("mainSource") && 
                     currentLayer.source.mainSource instanceof SolidSource){
                obj.type = 'solid';
                if(currentLayer.hasOwnProperty("scale"))
                    obj.scale = getProperty(currentLayer.scale);
            }else{
                continue;
            }
            obj.name = currentLayer.name;
            obj.translate = getProperty(currentLayer.position);
            if(currentLayer.hasOwnProperty("orientation"))
                obj.rotate = getProperty(currentLayer.orientation);
            
            data.push(obj);
        };
    }
    
    var jsData = {"layers" : data}    
    return JSON.stringify(jsData);
}

function getPrefs(){
    'use strict';
    
    data = {'width':myComp.width, 'height':myComp.height, 'pixelAspect':myComp.pixelAspect, 'duration':myComp.duration, 'frameRate':myComp.frameRate}
    
    return JSON.stringify({'prefs' : data});
    
    
}

function getCurrentComposition(data){
    var prefs = data.prefs;
    var samePrefs = false;
    var proj = app.project;
    if (!proj) {
        proj = app.newProject();
    }
    
    var myItemCollection = app.project.items;
    
    app.activeViewer.setActive();
    var myComp = proj.activeItem;
    if (!myComp) {
        myComp = myItemCollection.addComp(prefs.name, prefs.width, prefs.height, prefs.pixelAspect, prefs.duration, prefs.frameRate);
        myComp.openInViewer();
        samePrefs = true;
    } else {
        samePrefs = (
            myComp.height == prefs.height &&
            myComp.width == prefs.width &&
            myComp.pixelAspect == prefs.pixelAspect &&
            myComp.duration == prefs.duration &&
            myComp.frameRate == prefs.frameRate
           );
    }
    
    
    return JSON.stringify({'compName' : myComp.name, 'compId' : myComp.id, 'samePrefs':samePrefs})
}

function initAE(data) {
    'use strict';
    
    var prefix = 'MAYA_';
    
    function isObject(obj) {
        return obj && (typeof obj  === "object");
    }
    
    if (!isObject(data)) {
        return '';
    }
    
    function isArray(obj) {
        return isObject(obj) && (obj instanceof Array);
    }
    
    //Init or get active composition
    var proj = app.project;
    if (!proj) {
        proj = app.newProject();
    }
    var myComp;
    var myItemCollection = app.project.items;
    var allItems = '';
    var log = '' + String(data) + '\n';
    
    if (data.hasOwnProperty('compId')) {
        var ci;
        for (ci = 1; ci < myItemCollection.length + 1; ci++) {
            if (myItemCollection[ci].id === data.compId) {
                myComp = myItemCollection[ci];
            }
        }
    }
    
    if (!myComp) {
        return 'requestHandshake';
    }
    
    var myLayerCollection = myComp.layers;
    
    function setValueForProperty(property, objData) {
        
        if (objData.times.length > 0) {
            property.setValuesAtTimes(objData.times, objData.values);
        } else {
            var i;
            for (i = property.numKeys; 1 <= i; i--) {
                property.removeKey(i);
            }
            property.setValue(objData.values[0]);
        }
    }
    
    function setPoiValueForProperty(property, objData) {
        
        if (objData.times.length > 0) {
            property.setValuesAtTimes(objData.times, objData.poiValues);
        } else {
            var i;
            for (i = property.numKeys; 1 <= i; i--) {
                property.removeKey(i);
            }
            property.setValue(objData.poiValues[0]);
        }
    }
    
    function getObjWithUUID(uuid){
        var obj = null;
        for (var i = 1; i <= myLayerCollection.length; i++){
            if (myLayerCollection[i].comment === uuid){
                obj = myLayerCollection[i];
                break;
            }
        }
        return obj;
    }
    
    function getObjFromData(objData) {
        var obj = getObjWithUUID(objData.uuid);
        
        if (obj === null) {
            var type = objData.type.toLowerCase();
            if (type.indexOf("camera") > -1) {
                obj = myLayerCollection.addCamera(prefix + objData.name, [0, 0]);
                obj.autoOrient = AutoOrientType.NO_AUTO_ORIENT;
            } else if (type.indexOf("light") > -1) {
                obj = myLayerCollection.addLight(prefix + objData.name, [0, 0]);
                obj.autoOrient = AutoOrientType.NO_AUTO_ORIENT;
                if (objData.type.toLocaleLowerCase() === "pointLight".toLocaleLowerCase()) {
                    obj.lightType = LightType.POINT;
                } else if (objData.type.toLocaleLowerCase() === "spotLight".toLocaleLowerCase()) {
                    obj.lightType = LightType.SPOT;
                } else if (objData.type.toLocaleLowerCase() === "ambientLight".toLocaleLowerCase()) {
                    obj.lightType = LightType.AMBIENT;
                } else if (objData.type.toLocaleLowerCase() === "directionalLight".toLocaleLowerCase()) {
                    obj.lightType = LightType.PARALLEL;
                    obj.autoOrient = AutoOrientType.CAMERA_OR_POINT_OF_INTEREST;
                }
            } else if (type.indexOf("locator") > -1) {
                obj = myLayerCollection.addNull();
                obj.source.name = prefix + objData.name;
                obj.name = prefix + objData.name;
            } else if (type.indexOf("mesh") > -1) {
                obj = myLayerCollection.addSolid([0.7,0.7,0.7], '', 1,1,1);
                obj.source.name = prefix + objData.name;
                obj.name = prefix + objData.name;
                obj.threeDLayer = true;
                obj.collapseTransformation = true;
            }
            if (type.indexOf("mesh") < 0) 
                obj.threeDLayer = true;
            obj.comment = objData.uuid;
            
        }
        //obj.color.setValue([0.0, 170 / 255.0, 170 / 255.0]);
        return obj;
    }
    
    if (!isArray(data)) {
        if (data.hasOwnProperty('prefs')) {
            prefs = data.prefs;
            myComp.width = prefs.width;
            myComp.height = prefs.height;
            myComp.pixelAspect = prefs.pixelAspect;
            myComp.duration = prefs.duration;
            myComp.frameRate = prefs.frameRate;
        }
        
        if (data.hasOwnProperty('timeline')) {
            myComp.time = data.timeline;
        }
        
        if (data.hasOwnProperty('delete')) {
            var uuid = data.delete;
            var obj = getObjWithUUID(uuid);
            if (obj !== null){
                obj.locked = false;
                obj.remove();
            }
        }
        
        if (data.hasOwnProperty('rename')){
            var uuid = data['rename'][0];
            var name = data['rename'][1];
            var obj = getObjWithUUID(uuid);
            if (obj !== null){
                obj.locked = false;
                obj.name = prefix + name;
                if (obj.hasOwnProperty('source')){
                    obj.source.name = prefix + name;
                }
            }            
        }
        if (data.hasOwnProperty('data')) {
            data = data.data;
        } else {
            return;
        }
    }
    
    var i;
    for (i = 0; i < data.length; i++) {
        var objData = data[i];
        var obj = getObjFromData(objData);
        obj.locked = false;
        switch (objData.attr) {
        case "translate":
            if (objData.type.toLocaleLowerCase() !== "ambientLight".toLocaleLowerCase())
                setValueForProperty(obj.property("position"), objData);
            break;
        case "rotate":
            if (objData.type.toLocaleLowerCase() !== "ambientLight".toLocaleLowerCase() &&
                objData.type.toLocaleLowerCase() !== "pointLight".toLocaleLowerCase() &&
                objData.type.toLocaleLowerCase() !== "directionalLight".toLocaleLowerCase())
                setValueForProperty(obj.property("orientation"), objData);
            break;
        case "color":
            setValueForProperty(obj.color, objData);
            break;
        case "intensity":
            setValueForProperty(obj.intensity, objData);
            break;
        case "coneAngle":
            setValueForProperty(obj.coneAngle, objData);
            break;
        case "zoom":
            var j;
            for (j = 0; j < objData.values.length; j++) {
                objData.values[j] = Math.round(myComp.width * objData.values[j]);
            }
            setValueForProperty(obj.property("zoom"), objData);
            break;
        case "scale":
            if (objData.type.toLowerCase().indexOf("locator") > -1 ||
                objData.type.toLowerCase().indexOf("mesh") > -1) {
                setValueForProperty(obj.scale, objData);
            }
            break;
        default:
            break;
        }
        if (objData.type.toLocaleLowerCase() === "directionalLight".toLocaleLowerCase() && objData.hasOwnProperty('poiValues')){
            setPoiValueForProperty(obj.property("pointOfInterest"), objData);
        }
        obj.name = prefix + objData.name;
        if (obj.hasOwnProperty('source')){
            obj.source.name = prefix + objData.name;
        }
    }
}




